@@ -1,10 +1,12 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 #########################################################
-## Thai Language Toolkit : version  1.3.9
+## Thai Language Toolkit : version  1.4.1
 ## Chulalongkorn University
 ## written by Wirote Aroonmanakun
 ## Implemented :
+##      upgrade to gensim 4.0
+##      Corpus_build(DIR), W2V_train(Corpus), D2V_train(Corpus)
 ##      TNC_load(), TNC3g_load(), trigram_load(Filename), unigram(w1), bigram(w1,w2), trigram(w1,w2,w3)
 ##      collocates(w,SPAN,STAT,DIR,LIMIT, MINFQ) = [wa,wb,wc]
 ##      w2v_exist(w), similar_words(w), similarlity(w1,w2), outofgroup(WORDLST), analogy(w1,w2,w3) 
@@ -16,6 +18,9 @@ import math
 from collections import defaultdict
 from operator import itemgetter
 import gensim
+from gensim.models import Word2Vec, KeyedVectors
+from gensim.models.doc2vec import Doc2Vec, TaggedDocument
+
 import pickle
 from sklearn.decomposition import PCA
 try:
@@ -296,16 +301,129 @@ def compute_colloc(stat,w1,w2):
     return(value)
 
 #######################################################################
-### word2vec model created from TNC 3.0
+### Load Corpus from text files in a directory
+def Corpus_build(dir, sep="par", lang="th",stopword="y", filetype="txt"):
+    ThaiStopwords = ['ที่','การ','เป็น','ใน','ของ','มี','และ','ได้','ให้','ว่า','ไป','มา','ก็','ๆ','ความ','กับ','หรือ','อยู่','กัน','จาก','นี้','แต่','อย่าง','ด้วย','เขา','ขึ้น','นั้น','ผู้','ซึ่ง','ตาม','โดย','ยัง','เพื่อ','อีก','เมื่อ','ถึง','เพราะ','ออก','คือ','จึง','กว่า','ไว้','ถ้า','อะไร','ลง','แบบ','ทุก','อื่น','เช่น','น่า','สามารถ','แห่ง','นี่','ใคร','ใด','เอง','จะ','คง','เคย','อาจ','ต้อง','แล้ว','ครับ','คะ','ค่ะ','นะ','ไม่','ไม่ได้']
+    text_corpus = []
+#    count = defaultdict(int)
+
+    rootDir = dir
+
+    for dirName, subdirList, fileList in os.walk(rootDir):
+        for fname in sorted(fileList):
+            if not fname.endswith('.'+filetype):
+                continue
+            with open(dirName + '/' + fname,encoding='utf8') as f:
+                print(fname)
+                if sep == 'file':
+                    lines = f.readlines()
+                    doc = ' '.join(lines)
+                    doc = doc.replace('<s/>','')
+                    doc = doc.replace('\n','')
+                    wrdlst = re.split("[, |?:!\^]+",doc)
+                    if '' in wrdlst : wrdlst.remove('')
+                    if wrdlst != []:
+                        if lang =='th' and stopword == 'y':
+                            wrdlst_wo_sw = [w for w in wrdlst if not w in ThaiStopwords]
+                            text_corpus.append(wrdlst_wo_sw) 
+                        else:
+                            text_corpus.append(wrdlst) 
+                else:
+                    for doc in f.readlines():
+                        doc = doc.replace('\n','')
+                        doc = doc.replace('<s/>','')
+                        wrdlst = re.split("[, |?:!\^]+",doc)
+                        if '' in wrdlst : wrdlst.remove('')
+                        if wrdlst != []:
+                            if lang=='th' and stopword == 'y':
+                                wrdlst_wo_sw = [w for w in wrdlst if not w in ThaiStopwords]
+                                text_corpus.append(wrdlst_wo_sw) 
+                            else:
+                                text_corpus.append(wrdlst) 
+    return(text_corpus)
+
+def D2V_train(text_corpus):
+    documents = [TaggedDocument(doc, [i]) for i, doc in enumerate(text_corpus)]
+    #  building a model
+    model = Doc2Vec(vector_size=100, min_count=2, dm =1, epochs=30)
+    model.build_vocab(documents)
+    model.train(documents, total_examples=model.corpus_count, epochs=model.epochs)
+    return(model)
+
+def W2V_train(corpus, min=5):                       
+    model = Word2Vec(corpus,min_count=min)
+    return(model)
+#    model.save(outmodel+'.bin')
+
+## convert corpus data into word list data with frequency
+# if data is pos tagged  word_tag will be counted as one token        
+class Corpus:
+    def frequency(self,corpus):
+        word = {}
+        for line in corpus:
+            for w in line:
+                if w in word:
+                    word[w] +=1
+                else:
+                    word[w] =1
+        return(word)
+    def dispersion(self,corpus):
+        word = {}
+        for line in corpus:
+            for w in list(set(line)):
+                if w in word:
+                    word[w] +=1
+                else:
+                    word[w] =1
+        return(word)
+    def totalword(self,corpus):
+        total = 0
+        for line in corpus:
+            total += len(line)
+        return(total)
+
+## create object for comparing two word frequency lists
+class Xwordlist:
+    def intersect(self,A,B):
+        X = A.keys()
+        Y = B.keys()
+        return([value for value in X if value in Y])
+    def onlyA(self,A,B):
+        X = A.keys()
+        Y = B.keys()
+        return([value for value in X if value not in Y])
+    def onlyB(self,A,B):
+        X = A.keys()
+        Y = B.keys()
+        return([value for value in Y if value not in X])
+    def union(self,A,B):
+        X = A.keys()
+        Y = B.keys()
+        return(list(set(X)|set(Y))) 
+
+#######################################################################
+### word2vec model created from TNC 3.0 gensim 4.0
+
+def W2V_load(File="TNCc5model2.bin"):
+    global w2v_model
+
+    path = os.path.abspath(__file__)
+    ATA_PATH = os.path.dirname(path)
+    try:
+        w2v_model = Word2Vec.load(File,'rb')
+    except IOError:
+        w2v_model = Word2Vec.load(ATA_PATH +'/' +"TNCc5model2.bin",'rb')
+    return(1)    
+
 def w2v_load():
     global w2v_model
 
     path = os.path.abspath(__file__)
     ATA_PATH = os.path.dirname(path)
     try:
-        w2v_model = gensim.models.Word2Vec.load(ATA_PATH +'/' +"TNCc5model.bin")
+        w2v_model = Word2Vec.load(ATA_PATH +'/' +"TNCc5model2.bin",'rb')
     except IOError:
-        w2v_model = gensim.models.Word2Vec.load("TNCc5model.bin")
+        w2v_model = Word2Vec.load("TNCc5model2.bin",'rb')
     return(1)
 
 def w2v_exist(w):
@@ -314,13 +432,15 @@ def w2v_exist(w):
       w2v_model
     except NameError:
       w2v_load()
-    if w in list(w2v_model.wv.vocab):
+#    if w in list(w2v_model.wv.vocab):
+    if w in list(w2v_model.wv.index_to_key):
         return(True)
     else:
         return(False)
     
 def w2v(w):
-    if w in list(w2v_model.wv.vocab):
+#    if w in list(w2v_model.wv.vocab):
+    if w in list(w2v_model.wv.index_to_key):
         return(w2v_model.wv[w])
     else:
         return()
@@ -328,9 +448,9 @@ def w2v(w):
 def similarity(w1,w2):
     global w2v_model
     degree = ''
-    vocabs = list(w2v_model.wv.vocab)
+    vocabs = list(w2v_model.wv.index_to_key)
     if w1 in vocabs and w2 in vocabs:
-        degree = w2v_model.similarity(w1,w2)
+        degree = w2v_model.wv.similarity(w1,w2)
     else:
         degree = 0.    
     return(degree)
@@ -338,9 +458,9 @@ def similarity(w1,w2):
 def cosine_similarity(w1,w2):
     global w2v_model
     degree = ''
-    vocabs = list(w2v_model.wv.vocab)
+    vocabs = list(w2v_model.wv.index_to_key)
     if w1 in vocabs and w2 in vocabs:
-        degree = cosine_similarity = numpy.dot(w2v_model[w1], w2v_model[w2])/(numpy.linalg.norm(w2v_model[w1])* numpy.linalg.norm(w2v_model[w2]))
+        degree = cosine_similarity = numpy.dot(w2v_model.wv[w1], w2v_model.wv[w2])/(numpy.linalg.norm(w2v_model.wv[w1])* numpy.linalg.norm(w2v_model.wv[w2]))
     else:
         degree = 0.    
     return(degree)
@@ -348,8 +468,8 @@ def cosine_similarity(w1,w2):
 
 def similar_words(w1,n=10,cutoff=0.,score="n"):
     global w2v_model                
-    if w1 in list(w2v_model.wv.vocab):
-        out = w2v_model.most_similar(w1)
+    if w1 in list(w2v_model.wv.index_to_key):
+        out = w2v_model.wv.most_similar(w1)
         result = []
         ct = 0
         for (w,p) in out:
@@ -368,11 +488,11 @@ def outofgroup(wrdlst):
       w2v_model
     except NameError:
       w2v_load()
-    vocabs = list(w2v_model.wv.vocab)
+    vocabs = list(w2v_model.wv.index_to_key)
     for w in wrdlst:
         if w in vocabs:
             wrdlst1.append(w)
-    out = w2v_model.doesnt_match(wrdlst1)
+    out = w2v_model.wv.doesnt_match(wrdlst1)
     return(out)
 
 def w2v_diffplot(ww,wx,wy):
@@ -401,7 +521,7 @@ def w2v_dimplot(wrdlst):
       w2v_model
     except NameError:
       w2v_load()
-    vocabs = list(w2v_model.wv.vocab)
+    vocabs = list(w2v_model.wv.index_to_key)
     for w in wrdlst:
         if w in vocabs:
             wrdlst1.append(w)
@@ -425,7 +545,7 @@ def w2v_plot(wrdlst):
       w2v_model
     except NameError:
       w2v_load()
-    vocabs = list(w2v_model.wv.vocab)
+    vocabs = list(w2v_model.wv.index_to_key)
     for w in wrdlst:
         if w in vocabs:
             wrdlst1.append(w)
@@ -435,7 +555,7 @@ def w2v_plot(wrdlst):
         'size'   : 14}
     matplotlib.rc('font', **font)
 
-    X = w2v_model[wrdlst1]
+    X = w2v_model.wv[wrdlst1]
     pca = PCA(n_components=2)
     result = pca.fit_transform(X)
     # create a scatter plot of the projection
@@ -455,9 +575,9 @@ def analogy(w1,w2,w3, n=1):   #king - man + woman = queen
       w2v_model
     except NameError:
       w2v_load()
-    vocabs = list(w2v_model.wv.vocab)
+    vocabs = list(w2v_model.wv.index_to_key)
     if w1 in vocabs and w2 in vocabs and w3 in vocabs:    
-        return(w2v_model.most_similar(positive=[w3, w1], negative=[w2], topn=n))
+        return(w2v_model.wv.most_similar(positive=[w3, w1], negative=[w2], topn=n))
     else:
         return([])
 
@@ -493,10 +613,39 @@ def w2v_compare_color(wds):
 ############ END OF GENERAL MODULES ##########################################################################
 
 
+
+'''
 ## testing area
+
+# read all .wsg files in the directory
+pars = Corpus_build('/Users/macbook/Cloud/Dropbox/Corpus/Thaipublica/wsg',filetype="wsg")
+print(pars[0:3])
+x1 = Corpus()
+print(x1.totalword(pars))
+c1 = Corpus()
+c2 = Corpus()
+Xcomp = Comp_wordlist()
+print(Xcomp.onlyA(c1.frequency(pars[:100]),c2.frequency(pars[101:])))
+
+W2V_load('TNCc5model2.bin')
+print(similar_words('ขบ'))
+
+#train Doc2Vec model
+model = D2V_train(pars)
+#a new paragraph
+x = 'การ|ทดลอง|ภาค|สนาม|โดย|ทีม|ผู้|เชี่ยวชาญ|เรื่อง|การ|ให้|และ|การ|ช่วยเหลือ|ผู้|อื่น'.split('|')
+#create vector for that paragraph
+newvect = model.infer_vector(x)
+#find the most similar top 10 vectors
+sims = model.dv.most_similar([newvect])
+print(sims)
+#print using index [0][0], [0][1]
+print('1.',pars[int(sims[0][0])])
+print('2.',pars[int(sims[0][1])])
 #w2v_load()
 #w2v_plot("ผู้ชาย ผู้หญิง เก่ง ฉลาด สวย หล่อ".split(" "))
 #w2v_compare_color("ผู้ชาย ผู้หญิง เก่ง ฉลาด สวย หล่อ".split(" "))
 #w2v_plot(['เก็บรักษา', 'จัดเตรียม','เอา', 'รวบรวม', 'ซื้อ', 'สะสม'])
 #w2v_diffplot("เล็กน้อย","เล็ก", "น้อย")
-#w2v_plot(["แทรกซ้อน","แทรก", "ซ้อน"])
\ No newline at end of file
+#w2v_plot(["แทรกซ้อน","แทรก", "ซ้อน"])
+'''
\ No newline at end of file
