@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 #########################################################
-## Thai Language Toolkit : version  1.3.9
+## Thai Language Toolkit : version  1.4.1
 ## Chulalongkorn University
 ## word_segmentation, syl_segementation written by Wirote Aroonmanakun
 ## Implemented :
@@ -14,12 +14,14 @@
 import re
 import math
 import os
+import bz2
 from copy import deepcopy
 from collections import defaultdict
 from itertools import product
 
 #import pkg_resources
 import pickle
+import gzip
 from nltk.tag.perceptron import PerceptronTagger
 
 from sklearn.ensemble import RandomForestClassifier
@@ -153,7 +155,7 @@ def pos_load():
     tagger = PerceptronTagger(load=False)
     path = os.path.abspath(__file__)
     ATA_PATH = os.path.dirname(path)
-    filehandler = open(ATA_PATH +'/' + 'tnc-tagger.pick', 'rb') 
+    filehandler = open(ATA_PATH +'/' + 'pos_tagger.pick', 'rb') 
     tagger = pickle.load(filehandler)
 
 def change_tag(w,pos):
@@ -279,7 +281,7 @@ def ner_load():
           all_possible_transitions=False)
     path = os.path.abspath(__file__)
     ATA_PATH = os.path.dirname(path)
-    filehandler = open(ATA_PATH +'/' + 'ner-tagger.pick', 'rb') 
+    filehandler = open(ATA_PATH +'/' + 'ner_tagger.pick', 'rb') 
     ner_tagger = pickle.load(filehandler)
 
 def wrd_len(word):
@@ -425,16 +427,29 @@ def segment(txt):
             syl_seq += w+'~'
     return(output)
 
+# load a Pickle file and then compress it into a file with bz2 
+'''
+def useg_model_compress():
+    f = open('/Users/macbook/Cloud/Dropbox/prog/python/TLTK/tltk/sent_segment.pick','rb')
+    model = pickle.load(f)
+    f = bz2.BZ2File('/Users/macbook/Cloud/Dropbox/prog/python/TLTK/tltk/sent_segmentx.pklz','wb')
+    pickle.dump(model,f)
+    f.close()
+'''
+
 def useg_model_load():
     global useg_model
     useg_model = Pipeline([
     ('vectorizer', DictVectorizer(sparse=False)),
     ('classifier', RandomForestClassifier(n_jobs=2, random_state=0))])
-
+ 
     path = os.path.abspath(__file__)
     ATA_PATH = os.path.dirname(path)
-    filehandler = open(ATA_PATH +'/' + 'sent_segment_rfs.pick', 'rb') 
+#    print('XXXXX',ATA_PATH +'/' + 'sent_segment.pick')
+#    filehandler = open(ATA_PATH +'/' + 'sent_segment.pick', 'rb') 
+    filehandler = bz2.BZ2File(ATA_PATH +'/' + 'sent_segment.pklz', 'rb') 
     useg_model = pickle.load(filehandler)
+    return(useg_model)
     
 def untag(tagged_sentence):
     return [w for w, t in tagged_sentence]
@@ -1138,33 +1153,6 @@ def spell_variants(wrd,InDict='no',Karan='exclude'):
 
 
 #############################################################################################################
-###  Word Segmentation for Thai texts
-### Input = a paragraph of Thai texts
-def word_segmentX(Input):
-    global SegSep
-    global SSegSep
-    output = ""
-    out = ""
-    
-    Input = preprocess(Input)
-    sentLst = Input.split(SegSep)
-    for s in sentLst:
-#        print "s:",s
-        inLst = s.split(SSegSep)
-        for inp in inLst:
-            if inp == '': continue            
-#            print "inp:",inp
-#            objMatch = re.match(r"[a-zA-Z0-9\-\_\+\=\(\)\*\&\^\%\$\#\@\!\~\{\}\[\]\'\"\:\;\<\>\?\/\\\. ]+",inp)
-            objMatch = re.match(r"[^ก-์]+",inp)
-            if objMatch:
-                out = inp
-            else:
-                y = sylseg(inp)
-                out = wordseg(y)
-            output = output+out+WordSep
-        output = output+'<s/>'    ####write <s/> output for SegSep   
-    return(output)        
-
 
 
 ###################################################################
@@ -1221,65 +1209,6 @@ def wordseg_colloc(Input):
             return("<Fail>"+Input+"</Fail>")
         
 
-###################################################################
-###### Thai word segmentation using word similarities of adjacent pairs
-###### Input is a list of syllables
-###### also add each syllable as a potential word
-def wordseg_w2v(Input,spellchk):
-    global TDICT
-    global EndOfSent
-    global chart
-    global SegSep
-    global WordSep
-    global CollocSt
-    
-    part = []
-    chart = defaultdict(dict)
-    SylSep = '~'
-    outx = ""
-
-    chart.clear()
-    CollocSt = defaultdict(float)
-
-    tltk.corpus.w2v_load()
-    
-    part = Input.split(SegSep)
-    for inx in part:
-        SylLst = syl_segment(inx).split('~')
-        if SylLst[-1] == '<s/>': SylLst.pop()
-#        SylLst = inx.split(SylSep)
-        EndOfSent = len(SylLst)
-        ######### Gen unknown word by set each syllable as a potential word
-        if spellchk == 'yes' : gen_unknown_thaiw(SylLst)
-        for i in range(EndOfSent):
-            chart[i][i+1] = [SylLst[i]]
-        eng_abbr(SylLst)    
-        ############################################################
-        for i in range(EndOfSent):
-            for j in range(i,EndOfSent+1):
-                wrd = ''.join(SylLst[i:j])
-                if wrd in TDICT:
-                    chart[i][j] = [wrd]
-                    if j > i+1:   ### more than one syllable, compute St
-                        St = 0.0
-                        Stlst = []
-                        NoOfSyl = len(SylLst[i:j])
-                        for ii in range(i,j-1):
-#                            St = tltk.corpus.similarity(SylLst[ii],SylLst[ii+1])
-                            Stlst.append(tltk.corpus.similarity(SylLst[ii],SylLst[ii+1]))
-#                            print (SylLst[ii],SylLst[ii+1],xx)
-                        St = sum(Stlst)/len(Stlst)     
-                        CollocSt[(i,j)] = St    #### Compute STrength of the word
-#                        print(i,j,wrd,CollocSt[(i,j)])
-                    else:   ### one sylable word St = 0
-                        CollocSt[(i,j)] = 0.0
-        if chart_parse():
-#            return(chart[0][EndOfSent])
-            outx += WordSep.join(chart[0][EndOfSent])
-            return(outx)
-        else:
-            return("<Fail>"+Input+"</Fail>")
-
 ####################################################################
 #### Word segmentation using Dictionary lookup 
 #### Input = Thai string,  method = syl | word  output = n-best segmentations
@@ -1370,7 +1299,7 @@ def chartparse_mm_bn():
 
 ####################################################################
 #### Word segmentation using Maximal Matching (minimal word) approach
-#### Input = Thai string,  method = mm|colloc|ngram|w2v , 
+#### Input = Thai string,  method = mm|colloc|ngram, 
 ####   spellchk=yes|no 
 ######################################################################
 def word_segment(Input,method='colloc',spellchk='no'):
@@ -1395,8 +1324,6 @@ def word_segment(Input,method='colloc',spellchk='no'):
                     out = wordseg_mm(inp,method,spellchk)
                 elif method == 'colloc':
                     out =wordseg_colloc(inp)
-                elif method == 'w2v':
-                    out =wordseg_w2v(inp,spellchk)
 #                elif method == 'ngram':
 #                    out =wordseg_mm(inp,method,spellchk)
             output = output+out+WordSep
@@ -1529,7 +1456,6 @@ def compute_colloc(stat,w1,w2):
     global Count
     global BiType
     global Type
-    global NoTrigram
     global TotalWord
     global TotalLex
 
@@ -1802,7 +1728,6 @@ def prob_trisyl(SylLst):
     global Count
     global BiType
     global Type
-    global NoTrigram
     global TotalWord
     global TotalLex
     global SegSep
@@ -1837,7 +1762,6 @@ def prob_wb(w,pw1,pw2):
     global Count
     global BiType
     global Type
-    global NoTrigram
     global TotalWord
     global TotalLex
     
@@ -2175,10 +2099,25 @@ def initial():
 ############ END OF GENERAL MODULES 
 initial()
 
+#print(segment('สำนักงานเขตจตุจักรชี้แจงว่า ได้นำป้ายประกาศเตือนปลิงไปปักตามแหล่งน้ำ ในเขตอำเภอเมือง จังหวัดอ่างทอง หลังจากนายสุกิจ อายุ 65 ปี ถูกปลิงกัดแล้วไม่ได้ไปพบแพทย์'))
 
+'''
 ##########################################################################
 ## testing area
 
+
+f = open('/Users/macbook/Cloud/Dropbox/prog/python/TLTK/tltk/sent_segment.pick','rb')
+model = pickle.load(f)
+f = bz2.BZ2File('/Users/macbook/Cloud/Dropbox/prog/python/TLTK/tltk/sent_segmentx.pklz','wb')
+pickle.dump(model,f)
+f.close()
+print("Write gzip pickle")
+f = bz2.BZ2File('/Users/macbook/Cloud/Dropbox/prog/python/TLTK/tltk/sent_segmentx.pklz','rb')
+modelx = pickle.load(f)
+f.close()
+print("write load success")
+
+
 #import time
 
 #start = time.time()
@@ -2252,3 +2191,4 @@ initial()
 
 #print(word_segment('นายกรัฐมนตรีกล่าวกับคนขับรถประจำทางหลวงสายสองว่า อยากวิงวอนให้ใช้ความรอบคอบอย่าหลงเชื่อคำชักจูงหรือปลุกระดมของพวกหัวรุนแรงจากทางการไฟฟ้า'))
 
+'''
\ No newline at end of file
